# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

import os


VIDEO_SWITCH_CORE = {'sha1': "89effa43394b8dd9949ce422224bf3a1ad8f56b6",
                     'src': {'drive': "https://drive.google.com/file/d/"
                                      "13tUttASMoIombyMCkrFf_Gl64A5Qt30-/"
                                      "view?usp=sharing",
                             'github': "https://github.com/diegohdorta/"
                                       "models/raw/master/models/"
                                       "eIQVideoSwitchCore.zip"},
                     'window_title': "PyeIQ - Object Detection Switch Cores"}

CPU_IMG = os.path.join("/tmp", "cpu.jpg")
NPU_IMG = os.path.join("/tmp", "npu.jpg")


JPEG_EOF = b'\xff\xd9'

CPU = 0
NPU = 1

PAUSE = "kill -STOP {}"
RESUME = "kill -CONT {}"
RUN = "{} -a {}"



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

FIRE_CLASSIFICATION = {'image': "fire.jpg",
                       'model': "fire_detection.tflite",
                       'sha1': "2df946680459a3b20bd668f423dcdaa6b76a98b3",
                       'src': {'drive': "https://drive.google.com/file/d/"
                                        "1aDsAB2i9wl1xbkexXbiYfPawKUMYXfwH/"
                                        "view?usp=sharing",
                               'github': "https://github.com/diegohdorta/"
                                         "models/raw/master/models/"
                                         "eIQFireClassification.zip"},
                       'window_title': "PyeIQ - Fire Classification"}

OBJ_CLASSIFICATION = {'image': "cat.jpg",
                      'labels': "labels_mobilenet_quant_v1_224.txt",
                      'model': "mobilenet_v1_1.0_224_quant.tflite",
                      'sha1': "765c995e1d27c4a738f77cf13445e7b41306befc",
                      'src': {'drive': "https://drive.google.com/file/d/"
                                       "1yhQkJZwtuSyOvTOKi0ZVcIvZUR7GXv6z/"
                                       "view?usp=sharing",
                              'github': "https://github.com/diegohdorta/"
                                        "models/raw/master/models/"
                                        "eIQImageClassification.zip"},
                      'window_title': "PyeIQ - Image Classification"}

FIRE_MSG = {'fire': "Fire Detected!",
            'non-fire': "No Fire"}



# Copyright 2020 Michael Kuo
# Copyright 2020 NXP Semiconductors
#
# Parts of this file were copied from Michael Kuo's github repository 
# respecting its rights. All the modified parts below are according to
# MIT LICENSE terms.
#
# SPDX-License-Identifier:    MIT

import cv2
import numpy as np
from PIL import Image

from eiq.config import FONT
from eiq.engines.tflite.inference import TFLiteInterpreter
from eiq.modules.detection.config import LEFT, TOP, RIGHT, BOTTOM, CONFIDENCE, CLASSES, OBJ_DETECTION_YOLOV3
from eiq.modules.utils import DemoBase


class eIQObjectDetectionYOLOV3(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__, OBJ_DETECTION_YOLOV3)
        self.config = self.data['config']

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite as inference engine."
                "\n   * YOLO as default algorithm.\n")

    def usage(self, name=None, labels=True, model=True):
        super().usage(name=name, labels=labels, model=model)

    @staticmethod
    def expit(x):
        return 1 / (1 + np.exp(-x))

    @staticmethod
    def softmax(x):
        x = x - np.max(x)
        exp_x = np.exp(x)
        softmax_x = exp_x / np.sum(exp_x)
        return softmax_x

    def sort_results(self, e):
        return e[CONFIDENCE]

    def get_input_data(self, image):
        image = image.resize((self.interpreter.width(),
                              self.interpreter.height()))

        n = np.array(image, dtype='float32')
        n = n / 255.0

        return np.array([n], dtype='float32')

    @staticmethod
    def load_labels(labels_path):
        labels = []

        with open(labels_path, 'r') as lines:
            for line in lines.readlines():
                labels.append(line.rstrip())

        return labels

    def non_maximal_suppression(self, results):
        predictions = []

        if len(results):
            results.sort(reverse=True, key=self.sort_results)
            best_prediction = results.pop(0)
            predictions.append(best_prediction)

            while len(results) != 0:
                prediction = results.pop(0)
                overlaps = False

                for j in range(len(predictions)):
                    previous_prediction = predictions[j]

                    intersect_proportion = 0.0
                    primary = previous_prediction
                    secondary = prediction

                    if (primary[LEFT] < secondary[RIGHT]) \
                            and (primary[RIGHT] > secondary[LEFT]) \
                            and (primary[TOP] < secondary[BOTTOM]) \
                            and (primary[BOTTOM] > secondary[TOP]):
                        intersection = max(0, min(primary[RIGHT],
                                                  secondary[RIGHT])
                                                  - max(primary[LEFT],
                                           secondary[LEFT])) \
                                       * max(0, min(primary[BOTTOM],
                                                    secondary[BOTTOM])
                                                    - max(primary[TOP],
                                             secondary[TOP]))

                        main = np.abs(primary[RIGHT] - primary[LEFT]) \
                               * np.abs(primary[BOTTOM] - primary[TOP])
                        intersect_proportion = intersection / main

                    overlaps = overlaps or (intersect_proportion >
                                            self.config['overlap_threshold'])

                if not overlaps:
                    predictions.append(prediction)

        return predictions

    def check_result(self, data):
        results = []

        for row in range(self.config['grid_h']):
            for column in range(self.config['grid_w']):
                for box in range(self.config['boxes']):
                    item = data[row][column]
                    offset = (len(self.labels) + 5) * box

                    confidence = self.expit(item[offset + 4])

                    classes = item[offset + 5: offset + 5 + len(self.labels)]
                    classes = self.softmax(classes)

                    detected_class = np.argmax(classes)
                    max_class = classes[detected_class]

                    confidence_in_class = max_class * confidence

                    if confidence_in_class >= self.config['threshold']:
                        x_pos = (column + self.expit(item[offset])) \
                                * self.config['block_size']
                        y_pos = (row + self.expit(item[offset + 1])) \
                                * self.config['block_size']
                        w = (np.exp(item[offset + 2])
                             * self.config['anchors'][box][0]) \
                            * self.config['block_size']
                        h = (np.exp(item[offset + 3])
                             * self.config['anchors'][box][1]) \
                            * self.config['block_size']

                        left = max(0, x_pos - w / 2)
                        top = max(0, y_pos - h / 2)
                        right = min(self.interpreter.width() - 1,
                                    x_pos + w / 2)
                        bottom = min(self.interpreter.height() - 1,
                                     y_pos + h / 2)

                        results.append([left, top, right, bottom,
                                        confidence_in_class,
                                        self.labels[detected_class]])

        return self.non_maximal_suppression(results)

    def detect_object(self, frame):
        image = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
        image = self.get_input_data(image)
        self.interpreter.set_tensor(image)
        self.interpreter.run_inference()
        data = self.interpreter.get_tensor(0)[0]
        self.draw_rectangles(frame, self.check_result(data))
        self.overlay.draw_info(frame, self.model, self.media_src,
                               self.interpreter.inference_time)

        return frame

    def draw_rectangles(self, frame, predictions):
        width = frame.shape[1]
        height = frame.shape[0]
        width_ratio = width / self.interpreter.width()
        height_ratio = height / self.interpreter.height()

        for element in predictions:
            x1 = int(element[LEFT] * width_ratio)
            x2 = int(element[RIGHT] * width_ratio)
            y1 = int(element[TOP] * height_ratio)
            y2 = int(element[BOTTOM] * height_ratio)

            top = int(max(0, np.floor(y1 + 0.5).astype('int32')))
            left = int(max(0, np.floor(x1 + 0.5).astype('int32')))
            bottom = int(min(height, np.floor(y2 + 0.5).astype('int32')))
            right = int(min(width, np.floor(x2 + 0.5).astype('int32')))

            label_size = cv2.getTextSize(element[5], FONT['hershey'],
                                         FONT['size'], FONT['thickness'])[0]
            label_left = int(left - 3)
            label_top = int(top - 3)
            label_right = int(left + 3 + label_size[0])
            label_bottom = int(top - 5 - label_size[1])

            cv2.rectangle(frame, (left, top), (right, bottom), (0, 255, 0), 3)
            cv2.rectangle(frame, (label_left, label_top),
                          (label_right, label_bottom),
                          (0, 255, 0), cv2.FILLED)
            cv2.putText(frame, element[CLASSES], (left, top - 4),
                        FONT['hershey'], FONT['size'], FONT['color']['black'],
                        FONT['thickness'])

    def start(self):
        self.gather_data()
        self.labels = self.load_labels(self.labels)
        self.interpreter = TFLiteInterpreter(self.model)

    def run(self):
        self.start()
        self.run_inference(self.detect_object)



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

import os

import cv2
import numpy as np

from eiq.config import FONT, INF_TIME_MSG, MODEL_MSG, SRC_MSG, FPS_MSG

class OpenCVOverlay:
    def __init__(self):
        self.time = None
        self.fps = None

    def draw_fps(self, frame, fps):
        fps_msg = "{}: {}".format(FPS_MSG, fps)
        x_offset = frame.shape[1] - (cv2.getTextSize(fps_msg, FONT['hershey'],
                                                     0.8, 2)[0][0] + 10)
        cv2.putText(frame, fps_msg,
                    (x_offset, 25), FONT['hershey'], 0.8,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, fps_msg,
                    (x_offset, 25), FONT['hershey'], 0.8,
                    FONT['color']['white'], 1, cv2.LINE_AA)

    def draw_info(self, frame, model, src, time):
        model = os.path.basename(model)

        cv2.putText(frame, "{}: {}".format(INF_TIME_MSG, time),
                    (3, 20), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(INF_TIME_MSG, time),
                    (3, 20), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

        y_offset = frame.shape[0] - cv2.getTextSize(src, FONT['hershey'],
                                                    0.5, 2)[0][1]
        cv2.putText(frame, "{}: {}".format(SRC_MSG, src),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(SRC_MSG, src),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

        y_offset -= (cv2.getTextSize(model, FONT['hershey'], 0.5, 2)[0][1] + 3)
        cv2.putText(frame, "{}: {}".format(MODEL_MSG, model),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(MODEL_MSG, model),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

    def display_result(self, frame, model, src, time, result, labels, colors):
        width = frame.shape[1]
        height = frame.shape[0]

        for obj in result:
            pos = obj['pos']
            _id = obj['_id']

            x1 = int(pos[1] * width)
            x2 = int(pos[3] * width)
            y1 = int(pos[0] * height)
            y2 = int(pos[2] * height)

            top = max(0, np.floor(y1 + 0.5).astype('int32'))
            left = max(0, np.floor(x1 + 0.5).astype('int32'))
            bottom = min(height, np.floor(y2 + 0.5).astype('int32'))
            right = min(width, np.floor(x2 + 0.5).astype('int32'))

            label_size = cv2.getTextSize(labels[_id], FONT['hershey'],
                                         FONT['size'], FONT['thickness'])[0]
            label_rect_left = int(left - 3)
            label_rect_top = int(top - 3)
            label_rect_right = int(left + 3 + label_size[0])
            label_rect_bottom = int(top - 5 - label_size[1])

            cv2.rectangle(frame, (left, top), (right, bottom),
                          colors[int(_id) % len(colors)], 6)
            cv2.rectangle(frame, (label_rect_left, label_rect_top),
                          (label_rect_right, label_rect_bottom),
                          colors[int(_id) % len(colors)], -1)
            cv2.putText(frame, labels[_id], (left, int(top - 4)),
                        FONT['hershey'], FONT['size'],
                        FONT['color']['black'],
                        FONT['thickness'])
            self.draw_info(frame, model, src, time)

        return frame



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

import os
from socket import gethostname
from stat import S_IEXEC
import subprocess
import threading
import time

import cv2
import numpy as np

from eiq.apps.switch_video.config import *
from eiq.config import BASE_DIR, ZIP
from eiq.utils import args_parser, Downloader


class eIQVideoSwitchCore:
    def __init__(self):
        self.args = args_parser()
        self.pid = [0, 0]
        self.device = None
        self.class_name = self.__class__.__name__
        self.data = VIDEO_SWITCH_CORE

        self.base_dir = os.path.join(BASE_DIR, self.class_name)
        self.binary = os.path.join(self.base_dir, "bin", "video_switch_core")
        self.tmp_img = NPU_IMG

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite as inference engine."
                "\n   * Single Shot Detection as default algorithm.\n\n"
                "This application allows users to run an object detection demo\n"
                "using either CPU or GPU/NPU to perform inference on a video.\n")

    def gather_data(self):
        downloader = Downloader(self.args)
        downloader.retrieve_data(self.data['src'], self.class_name + ZIP,
                                 self.base_dir, self.data['sha1'], True)
        os.chmod(self.binary, S_IEXEC)

    def run_inference(self, device):
        self.pid[device] = subprocess.Popen(RUN.format(self.binary, device), shell=True).pid

    def get_device(self):
        hostname = gethostname()

        if "imx8mp" in hostname:
            self.device = "npu"
        elif "imx8" in hostname:
            self.device = "gpu"

    def pause_proc(self, dev):
        if self.pid[dev] > 0:
            proc = subprocess.Popen(PAUSE.format(self.pid[dev]), shell=True)
            proc.wait()

    def resume_proc(self, dev):
        if self.pid[dev] > 0:
            proc = subprocess.Popen(RESUME.format(self.pid[dev]), shell=True)
            proc.wait()

    def interruption(self):
        self.get_device()

        while True:
            interrupt = str(input("Choose: 'cpu' or '{}': ".format(self.device)))
            os.system("clear")

            if interrupt == "cpu":
                self.pause_proc(NPU)
                self.resume_proc(CPU)
                self.tmp_img = CPU_IMG
            elif interrupt == self.device:
                self.pause_proc(CPU)
                self.resume_proc(NPU)
                self.tmp_img = NPU_IMG
            else:
                print("Invalid option. Please, choose between 'cpu' or '{}'".format(self.device))

    def start_threads(self):
        input_thread = threading.Thread(target=self.interruption)
        cpu_inference_thread = threading.Thread(target=self.run_inference,
                                                args=(CPU,))
        npu_inference_thread = threading.Thread(target=self.run_inference,
                                                args=(NPU,))
        input_thread.daemon = True
        cpu_inference_thread.daemon = True
        npu_inference_thread.daemon = True
        cpu_inference_thread.start()
        time.sleep(3)
        proc = subprocess.Popen(PAUSE.format(self.pid[CPU]), shell=True)
        proc.wait()
        npu_inference_thread.start()
        time.sleep(5)
        input_thread.start()

    def start(self):
        os.environ['VSI_NN_LOG_LEVEL'] = "0"
        self.gather_data()
        self.start_threads()

    def run(self):
        self.start()

        while True:
            with open(self.tmp_img, 'rb') as f:
                bytes_img = f.read()

            if bytes_img.endswith(JPEG_EOF):
                image = cv2.imdecode(np.frombuffer(bytes_img, dtype=np.uint8),
                                     cv2.IMREAD_COLOR)
                cv2.imshow(self.data['window_title'], image)
                cv2.waitKey(1)



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

import os

import cv2
import numpy as np
from PIL import Image

from eiq.config import FONT
from eiq.engines.tflite.inference import TFLiteInterpreter
from eiq.modules.detection.config import FACIAL_EXPRESSION_DETECTION, FACE_EYES_DETECTION
from eiq.modules.utils import DemoBase
from eiq.utils import Timer


class eIQFacialExpressionDetection(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__,
                         FACIAL_EXPRESSION_DETECTION)

        self.face_cascade = None

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite and OpenCV as inference engines."
                "\n   * CNN and Haar Cascade as default algorithms.\n")

    def usage(self, name=None, labels=False, model=True):
        super().usage(name=name, labels=labels, model=model)

    def gather_data(self):
        super().gather_data()

        self.face_cascade = os.path.join(self.model_dir,
                                         self.data['face_cascade'])

    @staticmethod
    def preprocess_image(image, x, y, w, h):
        image = image[y:y+h, x:x+w]
        image = Image.fromarray(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
        image = image.resize((200, 200))
        image = np.expand_dims(image, axis=0)

        if np.max(image) > 1:
            image = image / 255.0

        return image.astype(np.float32)

    def detect_facial_expression(self, image, x, y, w, h):
        image = self.preprocess_image(image, x, y, w, h)

        self.interpreter.set_tensor(image)
        self.interpreter.run_inference()

        results = self.interpreter.get_tensor(0)
        classes = int(np.argmax(results, axis=1))

        if 0 <= classes <= 5:
            return self.data['expressions'][classes]
        else:
            return 'surprise'

    def detect_face(self, frame):
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        faces = self.face_cascade.detectMultiScale(gray,
                                                   scaleFactor=1.1,
                                                   minNeighbors=5,
                                                   minSize=(150, 150))

        for (x, y, w, h) in faces:
            expression = self.detect_facial_expression(gray, x, y, w, h)
            cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)
            cv2.putText(frame, expression, (x, y-5), FONT['hershey'],
                        0.7, (0, 0, 255), 2, cv2.LINE_AA)

        self.overlay.draw_info(frame, self.model, self.media_src,
                               self.interpreter.inference_time)

        return frame

    def start(self):
        self.gather_data()
        self.face_cascade = cv2.CascadeClassifier(self.face_cascade)
        self.interpreter = TFLiteInterpreter(self.model)

    def run(self):
        self.start()
        self.run_inference(self.detect_face)


class eIQFaceAndEyesDetection(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__, FACE_EYES_DETECTION)
        self.timer = Timer()

        self.eye_cascade = None
        self.face_cascade = None

    @staticmethod
    def description():
        return ("This demo uses:\n   * OpenCV as inference engine."
                "\n   * Haar Cascade as default algorithm.\n")

    def usage(self, name=None, labels=False, model=False):
        super().usage(name=name, labels=labels, model=model)

    def gather_data(self):
        super().gather_data()

        self.eye_cascade = os.path.join(self.model_dir,
                                        self.data['eye_cascade'])
        self.face_cascade = os.path.join(self.model_dir,
                                         self.data['face_cascade'])

    def detect_face(self, frame):
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

        with self.timer.timeit():
            faces = self.face_cascade.detectMultiScale(gray, 1.3, 5)

        for (x, y, w, h) in faces:
            cv2.rectangle(frame, (x, y), (x+w, y+h), (255, 0, 0), 2)
            roi_gray = gray[y:y+h, x:x+w]
            roi_color = frame[y:y+h, x:x+w]

            eyes = self.eye_cascade.detectMultiScale(roi_gray)
            for (ex, ey, ew, eh) in eyes:
                cv2.rectangle(roi_color, (ex, ey), (ex+ew, ey+eh),
                              (0, 255, 0), 2)

        self.overlay.draw_info(frame, "haarcascade_frontalface_default.xml",
                               self.media_src, self.timer.time)

        return frame

    def start(self):
        self.gather_data()
        self.eye_cascade = cv2.CascadeClassifier(self.eye_cascade)
        self.face_cascade = cv2.CascadeClassifier(self.face_cascade)

    def run(self):
        self.start()
        self.run_inference(self.detect_face)



# Copyright 2019 Google LLC
# Copyright 2020 NXP Semiconductors
#
# This file was copied from Google LLC respecting its rights. All the modified
# parts below are according to Google LLC's LICENSE terms.
#
# SPDX-License-Identifier:    Apache-2.0
#
# References:
# https://github.com/ggsonic/posenet-tflite
# https://github.com/mikeshihyaolin/tf-pose-estimation

from enum import Enum
import math

import cv2
import numpy as np
from PIL import Image, ImageDraw

from eiq.engines.tflite.inference import TFLiteInterpreter
from eiq.modules.detection.config import POSE_DETECTION
from eiq.modules.utils import DemoBase


class eIQPoseDetection(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__, POSE_DETECTION)
        self.config = self.data['config']

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite as inference engine."
                "\n   * CNN as default algorithm.\n")

    def usage(self, name=None, labels=False, model=True):
        super().usage(name=name, labels=labels, model=model)

    class BodyPart(Enum):
        NOSE = 0,
        LEFT_EYE = 1,
        RIGHT_EYE = 2,
        LEFT_EAR = 3,
        RIGHT_EAR = 4,
        LEFT_SHOULDER = 5,
        RIGHT_SHOULDER = 6,
        LEFT_ELBOW = 7,
        RIGHT_ELBOW = 8,
        LEFT_WRIST = 9,
        RIGHT_WRIST = 10,
        LEFT_HIP = 11,
        RIGHT_HIP = 12,
        LEFT_KNEE = 13,
        RIGHT_KNEE = 14,
        LEFT_ANKLE = 15,
        RIGHT_ANKLE = 16,

    class Position:
        def __init__(self):
            self.x = 0
            self.y = 0

    class KeyPoint:
        def __init__(self):
            self.bodyPart = eIQPoseDetection.BodyPart.NOSE
            self.position = eIQPoseDetection.Position()
            self.score = 0.0

    class Person:
        def __init__(self):
            self.keyPoints = []
            self.score = 0.0

    @staticmethod
    def sigmoid(x):
        return 1. / (1. + math.exp(-x))

    def estimate_pose(self, image):
        w, h = image.size
        image = image.resize((self.interpreter.width(),
                              self.interpreter.height()))
        image = np.expand_dims(image, axis=0)

        if self.interpreter.dtype() == np.float32:
            image = (np.float32(image)-self.config['mean']) / self.config['std']

        self.interpreter.set_tensor(image)
        self.interpreter.run_inference()

        heat_maps = self.interpreter.get_tensor(0)
        offset_maps = self.interpreter.get_tensor(1)

        height = len(heat_maps[0])
        width = len(heat_maps[0][0])
        num_key_points = len(heat_maps[0][0][0])
        key_point_positions = [[0] * 2 for i in range(num_key_points)]

        for key_point in range(num_key_points):
            max_val = heat_maps[0][0][0][key_point]
            max_row = 0
            max_col = 0
            for row in range(height):
                for col in range(width):
                    heat_maps[0][row][col][key_point] = self.sigmoid(heat_maps[0][row][col][key_point])
                    if heat_maps[0][row][col][key_point] > max_val:
                        max_val = heat_maps[0][row][col][key_point]
                        max_row = row
                        max_col = col
            key_point_positions[key_point] = [max_row, max_col]

        x_coords = [0] * num_key_points
        y_coords = [0] * num_key_points
        confidence_scores = [0] * num_key_points

        for i, position in enumerate(key_point_positions):
            position_y = int(key_point_positions[i][0])
            position_x = int(key_point_positions[i][1])
            y_coords[i] = (position[0] / float(height - 1)*h
                           + offset_maps[0][position_y][position_x][i])
            x_coords[i] = (position[1] / float(width - 1)*w
                           + offset_maps[0][position_y][position_x][i + num_key_points])
            confidence_scores[i] = heat_maps[0][position_y][position_x][i]

        person = self.Person()
        key_point_list = []
        total_score = 0

        for i in range(num_key_points):
            key_point = self.KeyPoint()
            key_point_list.append(key_point)

        for i, body_part in enumerate(self.BodyPart):
            key_point_list[i].bodyPart = body_part
            key_point_list[i].position.x = x_coords[i]
            key_point_list[i].position.y = y_coords[i]
            key_point_list[i].score = confidence_scores[i]
            total_score += confidence_scores[i]

        person.keyPoints = key_point_list
        person.score = total_score / num_key_points

        return person

    def detect_pose(self, frame):
        frame = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
        draw = ImageDraw.Draw(frame)
        person = self.estimate_pose(frame)

        body_joints = [[self.BodyPart.LEFT_WRIST, self.BodyPart.LEFT_ELBOW],
                       [self.BodyPart.LEFT_ELBOW, self.BodyPart.LEFT_SHOULDER],
                       [self.BodyPart.LEFT_SHOULDER, self.BodyPart.RIGHT_SHOULDER],
                       [self.BodyPart.RIGHT_SHOULDER, self.BodyPart.RIGHT_ELBOW],
                       [self.BodyPart.RIGHT_ELBOW, self.BodyPart.RIGHT_WRIST],
                       [self.BodyPart.LEFT_SHOULDER, self.BodyPart.LEFT_HIP],
                       [self.BodyPart.LEFT_HIP, self.BodyPart.RIGHT_HIP],
                       [self.BodyPart.RIGHT_HIP, self.BodyPart.RIGHT_SHOULDER],
                       [self.BodyPart.LEFT_HIP, self.BodyPart.LEFT_KNEE],
                       [self.BodyPart.LEFT_KNEE, self.BodyPart.LEFT_ANKLE],
                       [self.BodyPart.RIGHT_HIP, self.BodyPart.RIGHT_KNEE],
                       [self.BodyPart.RIGHT_KNEE, self.BodyPart.RIGHT_ANKLE]]

        for line in body_joints:
            if person.keyPoints[line[0].value[0]].score > self.config['confidence'] \
                    and person.keyPoints[line[1].value[0]].score > self.config['confidence']:
                start_point_x = int(person.keyPoints[line[0].value[0]].position.x)
                start_point_y = int(person.keyPoints[line[0].value[0]].position.y)
                end_point_x = int(person.keyPoints[line[1].value[0]].position.x)
                end_point_y = int(person.keyPoints[line[1].value[0]].position.y)
                draw.line((start_point_x, start_point_y, end_point_x, end_point_y),
                          fill=(255, 255, 0), width=3)

        for key_point in person.keyPoints:
            if key_point.score > self.config['confidence']:
                left_top_x = int(key_point.position.x) - 5
                left_top_y = int(key_point.position.y) - 5
                right_bottom_x = int(key_point.position.x) + 5
                right_bottom_y = int(key_point.position.y) + 5
                draw.ellipse((left_top_x, left_top_y, right_bottom_x, right_bottom_y),
                             fill=(0, 128, 0), outline=(255, 255, 0))

        frame = cv2.cvtColor(np.array(frame), cv2.COLOR_RGB2BGR)
        self.overlay.draw_info(frame, self.model, self.media_src, self.interpreter.inference_time)

        return frame

    def start(self):
        self.gather_data()
        self.interpreter = TFLiteInterpreter(self.model)

    def run(self):
        self.start()
        self.run_inference(self.detect_pose)



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

LEFT = 0
TOP = 1
RIGHT = 2
BOTTOM = 3
CONFIDENCE = 4
CLASSES = 5

FACE_EYES_DETECTION = {'eye_cascade': "haarcascade_eye.xml",
                       'face_cascade': "haarcascade_frontalface_default.xml",
                       'image': "grace_hopper.jpg",
                       'sha1': "79ac7f90076ed5e2724e791bc27b3840ef63eb11",
                       'src': {'drive': "https://drive.google.com/file/d/"
                                        "1HPWV4W4FnrfG14tnZF7bugkgoCmMpM-p/"
                                        "view?usp=sharing",
                               'github': "https://github.com/diegohdorta/"
                                         "models/raw/master/models/"
                                         "eIQFaceAndEyesDetection.zip"},
                       'window_title': "PyeIQ - Face and Eyes Detection"}

FACIAL_EXPRESSION_DETECTION = {'expressions': ['anger', 'disgust', 'fear', 'happiness',
                                               'neutral', 'sadness'],
                               'face_cascade': "haarcascade_frontalface_default.xml",
                               'image': "grace_hopper.jpg",
                               'model': "model.tflite",
                               'sha1': "d112d8e378f8457a48b66aea80bc4e714e8e2e41",
                               'src': {'drive': "https://drive.google.com/file/d/"
                                                "15VTXVJ7GwS_Rr2vvl9xve9UAcFO3LFzG/"
                                                "view?usp=sharing",
                                       'github': "https://github.com/diegohdorta/"
                                                 "models/raw/master/models/"
                                                 "eIQEmotionsDetection.zip"},
                               'window_title': "PyeIQ - Facial Expression Detection"}

OBJ_DETECTION = {'image': "bus.jpg",
                 'labels': "coco_labels.txt",
                 'model': "detect.tflite",
                 'sha1': "73b8bb0749f275c10366553bab6f5f313230c527",
                 'src': {'drive': "https://drive.google.com/file/d/"
                                  "1xdjdKPOH2PFPStbU2K2TEuFJN8Las3ag/"
                                  "view?usp=sharing",
                         'github': "https://github.com/diegohdorta/"
                                   "models/raw/master/models/"
                                   "eIQObjectDetection.zip"},
                 'window_title': "PyeIQ - Object Detection"}

OBJ_DETECTION_DNN = {'caffe': "MobileNetSSD_deploy.caffemodel",
                     'config': {'dims': 300,
                                'normalize': 127.5,
                                'scale': 0.009718,
                                'threshold': 0.2},
                     'image': "dog.jpg",
                     'labels': "labels.txt",
                     'proto': "MobileNetSSD_deploy.prototxt",
                     'sha1': "f9894307c83f8ddec91af76b8cd6f3dc07196dc0",
                     'src': {'drive': "https://drive.google.com/file/d/"
                                      "1_qeq3CxK-xhrVX4qdsmnWQ_dwlMCWF76/"
                                      "view?usp=sharing",
                             'github': "https://github.com/diegohdorta/"
                                       "models/raw/master/models/"
                                       "object_detection_image.zip"},
                     'window_title': "PyeIQ - Object Detection DNN"}

OBJ_DETECTION_GST = {'labels': "coco_labels.txt",
                     'model': "mobilenet_ssd_v2_coco_quant_postprocess.tflite",
                     'sha1': "4736e758d8d626047df7cd1b3c38c72e77fd32ee",
                     'src': {'drive': "https://drive.google.com/file/d/"
                                      "1l2gKkPIPPPI0Q4QrVBXgSOkQUDVHogOi/"
                                      "view?usp=sharing",
                             'github': "https://github.com/diegohdorta/"
                                       "models/raw/master/models/"
                                       "eIQObjectDetectionCVGST.zip"},
                     'window_title': "PyeIQ - Object Detection OpenCV"}

OBJ_DETECTION_YOLOV3 = {'config': {'anchors': [[0.57273, 0.677385],
                                               [1.87446, 2.06253],
                                               [3.33843, 5.47434],
                                               [7.88282, 3.52778],
                                               [9.77052, 9.16828]],
                                   'block_size': 32,
                                   'boxes': 5,
                                   'grid_h': 13,
                                   'grid_w': 13,
                                   'overlap_threshold': 0.2,
                                   'threshold': 0.3},
                        'image': "example.jpg",
                        'labels': "labels.txt",
                        'model': "tiny_yolov3.tflite",
                        'sha1': "406438b9a5a530f6f6874341219a749e4f209b6e",
                        'src': {'drive': "https://drive.google.com/file/d/"
                                         "1gzUVbyDZrgAFDyZG3aRpnsClufL9Q7Hx/"
                                         "view?usp=sharing",
                                'github': "https://github.com/diegohdorta/"
                                          "models/raw/master/models/"
                                          "eIQObjectDetectionYOLOV3.zip"},
                        'window_title': "PyeIQ - Object Detection YOLOV3"}

POSE_DETECTION = {'config': {'mean': 127.5,
                             'std': 127.5,
                             'confidence': 0.4},
                  'image': "example.jpg",
                  'model': "posenet_mobilenet_v1_100_257x257"
                           "_multi_kpt_stripped.tflite",
                  'sha1': "fd4fa2ebdaeb43a8f6633888456bfb068e04a5ae",
                  'src': {'drive': "https://drive.google.com/file/d/"
                                   "1EoAGgsooOQbPYV7Okgt6VbcrdMEHZRkW/"
                                   "view?usp=sharing",
                          'github': "https://github.com/diegohdorta/models/"
                                    "raw/master/models/posenet_model.zip"},
                  'window_title': "PyeIQ - Pose Detection"}


# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

from eiq.apps.switch_image.switch_image import eIQSwitchLabelImage
from eiq.apps.switch_video.switch_video import eIQVideoSwitchCore
from eiq.modules.classification.classification_tflite import eIQFireClassificationTFLite, eIQObjectClassificationTFLite
from eiq.modules.classification.classification_armnn import eIQFireClassificationArmNN
from eiq.modules.detection.facial_detection import eIQFaceAndEyesDetection, eIQFacialExpressionDetection
from eiq.modules.detection.object_detection_ssd import eIQObjectDetection, eIQObjectDetectionDNN
from eiq.modules.detection.object_detection_yolo import eIQObjectDetectionYOLOV3
from eiq.modules.detection.pose_detection import eIQPoseDetection


APPS = {'switch_image': eIQSwitchLabelImage,
        'switch_video': eIQVideoSwitchCore}

DEMOS = {'face_and_eyes_detection': eIQFaceAndEyesDetection,
         'facial_expression_detection': eIQFacialExpressionDetection,
         'fire_classification_armnn': eIQFireClassificationArmNN,
         'fire_classification_tflite': eIQFireClassificationTFLite,
         'object_classification_tflite': eIQObjectClassificationTFLite,
         'object_detection_dnn': eIQObjectDetectionDNN,
         'object_detection_tflite': eIQObjectDetection,
         'object_detection_yolov3': eIQObjectDetectionYOLOV3,
         'pose_detection': eIQPoseDetection}



# Copyright 2020 NXP Semiconductors
# SPDX-License-Identifier: BSD-3-Clause

def v4l2_camera_pipeline(width, height, device, frate,
                         leaky="leaky=downstream max-size-buffers=1",
                         sync="sync=false emit-signals=true drop=true "\
                              "max-buffers=1"):

    return (("v4l2src device={} ! video/x-raw,width={},height={},framerate={} "\
             "! queue {} ! videoconvert ! appsink {}").format(device, width,
                                                              height, frate,
                                                              leaky, sync))


def v4l2_video_pipeline(device, leaky="leaky=downstream max-size-buffers=1",
                        sync="sync=false drop=True max-buffers=1 "\
                             "emit-signals=True max-lateness=8000000000"):

    return (("filesrc location={} ! qtdemux name=d d.video_0 ! decodebin ! queue {} ! queue ! imxvideoconvert_g2d ! videoconvert ! "\
             "appsink {}").format(device, leaky, sync))
